﻿using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetPickupRequestDetails
{
    class GetPickupRequestDetails
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_Pickup_DotNet_Samples\SOAP\pickup\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            PickupRequestPortTypeClient server = new PickupRequestPortTypeClient("PickupRequestPort", "https://ct.soa-gw.canadapost.ca/enab/soap/pickuprequest");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getpickuprequestdetailsrequest request = new getpickuprequestdetailsrequest();

            request.customernumber = mailedBy;
            request.requestid = "0074698052";
            request.locale = "FR";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getpickuprequestdetailsresponse response = server.GetPickupRequestDetails(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(PickupRequestDetailedInfoType))
                {
                    PickupRequestDetailedInfoType pickupDetailsType = (PickupRequestDetailedInfoType)response.Item;
                    PickupRequestHeaderType header = pickupDetailsType.pickuprequestheader;
                    PickupRequestResponseDetailsType details = pickupDetailsType.pickuprequestdetails;

                    responseAsString += "Request Id: " + header.requestid + "\r\n";
                    responseAsString += "Request Status: " + header.requeststatus + "\r\n";
                    responseAsString += "Pickup Type: " + header.pickuptype + "\r\n";
                    responseAsString += "Request Date: " + header.requestdate.ToString("yyyy-MM-dd") + "\r\n";
                    responseAsString += "Pickup Volume: " + details.pickupvolume + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
